#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <curl/curl.h>

#include "requests.h"

void requestsInit(void) {
    curl_global_init(CURL_GLOBAL_DEFAULT);
}

void requestsCleanup(void) {
    curl_global_cleanup();
}

GetRequestResult requestsGet(const char* url) {
    printf("requestGet(%s)\n", url);

    GetRequestResult result = {
        .errorCode = -1,
        .data = NULL,
    };

    CURL *handle = curl_easy_init();
    CURLcode err;

    err = curl_easy_setopt(handle, CURLOPT_URL, url);
    if (err) {
        printf("Failed to set url\n");
        return result;
    }
    err = curl_easy_setopt(handle, CURLOPT_WRITEFUNCTION, _getRequestCb);
    if (err) {
        printf("Failed to set callback function\n");
        return result;
    }
    err = curl_easy_setopt(handle, CURLOPT_WRITEDATA, &result);
    if (err) {
        printf("Failed to set callback data\n");
        return result;
    }
    result.errorCode = 0;
    result.size = 0;
    result.capacity = 1024;
    result.data = calloc(1024, sizeof(char));
    err = curl_easy_perform(handle);
    if (err) {
        printf("Error performing request\n");
        result.errorCode = -1;
        result.size = 0;
        free(result.data);
        result.data = NULL;
        return result;
    }
    return result;
}

size_t _getRequestCb(char *buffer, size_t size, size_t nmemb, GetRequestResult *result) {
    size_t numBytes = size * nmemb;
    while (result->size + numBytes > result->capacity + 1 /*extra byte for null terminator*/) {
        size_t newCapacity = result->capacity * 2;
        char *newData = calloc(newCapacity, sizeof(char));
        memcpy(newData, result->data, result->size);
        free(result->data);
        result->data = newData;
        result->capacity = newCapacity;
    }
    memcpy(result->data + result->size, buffer, numBytes);
    result->size += numBytes;
    result->data[result->size] = '\0';
    return numBytes;
}
